﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using HeuristicLab.Optimization;
using HeuristicLab.PluginInfrastructure;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.Instances;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Common;
using HeuristicLab.Parameters;
using HeuristicLab.Data;

namespace HeuristicLab.Problems.PTSP {
  [Item("Analytical Probabilistic Traveling Salesman Problem", "Represents an analytical Probabilistic Traveling Salesman Problem.")]
  [Creatable("Problems")]
  [StorableClass]
  public sealed class AnalyticalProbabilisticTravelingSalesmanProblem : ProbabilisticTravelingSalesmanProblem, IStorableContent,
  IProblemInstanceConsumer<TSPData> {

    [StorableConstructor]
    private AnalyticalProbabilisticTravelingSalesmanProblem(bool deserializing) : base(deserializing) { }
    private AnalyticalProbabilisticTravelingSalesmanProblem(AnalyticalProbabilisticTravelingSalesmanProblem original, Cloner cloner)
      : base(original, cloner) {
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new AnalyticalProbabilisticTravelingSalesmanProblem(this, cloner);
    }

    public override double Evaluate(Individual individual, IRandom random) {
      Permutation p = individual.Permutation();
      // Analytical evaluation
      double firstSum = 0;
      for (int i = 0; i < p.Length - 1; i++) {
        for (int j = i + 1; j < p.Length - 1; j++) {
          double sum1 = DistanceMatrix[p[i], p[j]] * ProbabilityMatrix[0, p[i]] * ProbabilityMatrix[0, p[j]];
          for (int k = i + 1; k < j; k++) {
            sum1 = sum1 * (1 - ProbabilityMatrix[0, p[k]]);
          }
          firstSum += sum1;
        }
      }
      double secondSum = 0;
      for (int j = 0; j < p.Length - 1; j++) {
        for (int i = 0; i < j; i++) {
          double sum2 = DistanceMatrix[p[j], p[i]] * ProbabilityMatrix[0, p[i]] * ProbabilityMatrix[0, p[j]];
          for (int k = j + 1; k < p.Length - 1; k++) {
            sum2 = sum2 * (1 - ProbabilityMatrix[0, p[k]]);
          }
          for (int k = 1; k < i; k++) {
            sum2 = sum2 * (1 - ProbabilityMatrix[0, p[k]]);
          }
          secondSum += sum2;
        }
      }
      return firstSum + secondSum;
    }

    public AnalyticalProbabilisticTravelingSalesmanProblem() {
      Operators.Add(new PTSPInversionMovePathEvaluator());
    }

    public override void Load(TSPData data) {
      base.Load(data);
      // Compute A and B matrices
    }

  }
}
